"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const context_1 = require("../Form/context");
const useDebounce_1 = __importDefault(require("../../../hooks/useDebounce"));
require("./index.scss");
const useFieldType = (options) => {
    const { path, validate, enableDebouncedValue, disableFormData, ignoreWhileFlattening, stringify, } = options;
    const formContext = context_1.useForm();
    const submitted = context_1.useFormSubmitted();
    const processing = context_1.useFormProcessing();
    const modified = context_1.useFormModified();
    const { dispatchFields, getField, setModified, } = formContext;
    const [internalValue, setInternalValue] = react_1.useState(undefined);
    // Debounce internal values to update form state only every 60ms
    const debouncedValue = useDebounce_1.default(internalValue, 120);
    // Get field by path
    const field = getField(path);
    const fieldExists = Boolean(field);
    const initialValue = field?.initialValue;
    // Valid could be a string equal to an error message
    const valid = (field && typeof field.valid === 'boolean') ? field.valid : true;
    const showError = valid === false && submitted;
    // Method to send update field values from field component(s)
    // Should only be used internally
    const sendField = react_1.useCallback(async (valueToSend) => {
        const fieldToDispatch = {
            path,
            value: valueToSend,
            valid: false,
            errorMessage: undefined,
            stringify: false,
            disableFormData: false,
            ignoreWhileFlattening: false,
            initialValue: undefined,
            validate: undefined,
        };
        const validationResult = typeof validate === 'function' ? await validate(valueToSend) : true;
        if (typeof validationResult === 'string') {
            fieldToDispatch.errorMessage = validationResult;
            fieldToDispatch.valid = false;
        }
        else {
            fieldToDispatch.valid = validationResult;
        }
        fieldToDispatch.stringify = stringify;
        fieldToDispatch.disableFormData = disableFormData;
        fieldToDispatch.ignoreWhileFlattening = ignoreWhileFlattening;
        fieldToDispatch.initialValue = initialValue;
        fieldToDispatch.validate = validate;
        dispatchFields(fieldToDispatch);
    }, [path, dispatchFields, validate, disableFormData, ignoreWhileFlattening, initialValue, stringify]);
    // Method to return from `useFieldType`, used to
    // update internal field values from field component(s)
    // as fast as they arrive. NOTE - this method is NOT debounced
    const setValue = react_1.useCallback((e) => {
        const val = (e && e.target) ? e.target.value : e;
        if (!ignoreWhileFlattening && !modified) {
            setModified(true);
        }
        setInternalValue(val);
    }, [setModified, modified, ignoreWhileFlattening]);
    react_1.useEffect(() => {
        setInternalValue(initialValue);
    }, [initialValue]);
    // The only time that the FORM value should be updated
    // is when the debounced value updates. So, when the debounced value updates,
    // send it up to the form
    const valueToSend = enableDebouncedValue ? debouncedValue : internalValue;
    react_1.useEffect(() => {
        if (valueToSend !== undefined || !fieldExists) {
            sendField(valueToSend);
        }
    }, [valueToSend, sendField, fieldExists]);
    return {
        ...options,
        showError,
        errorMessage: field?.errorMessage,
        value: internalValue,
        formSubmitted: submitted,
        formProcessing: processing,
        setValue,
    };
};
exports.default = useFieldType;
