"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mkdirp_1 = __importDefault(require("mkdirp"));
const path_1 = __importDefault(require("path"));
const crypto_1 = __importDefault(require("crypto"));
const executeAccess_1 = __importDefault(require("../../auth/executeAccess"));
const removeInternalFields_1 = __importDefault(require("../../utilities/removeInternalFields"));
const errors_1 = require("../../errors");
const imageResizer_1 = __importDefault(require("../../uploads/imageResizer"));
const getSafeFilename_1 = __importDefault(require("../../uploads/getSafeFilename"));
const getImageSize_1 = __importDefault(require("../../uploads/getImageSize"));
const isImage_1 = __importDefault(require("../../uploads/isImage"));
const sendVerificationEmail_1 = __importDefault(require("../../auth/sendVerificationEmail"));
async function create(incomingArgs) {
    const { performFieldOperations, config, emailOptions } = this;
    let args = incomingArgs;
    // /////////////////////////////////////
    // beforeOperation - Collection
    // /////////////////////////////////////
    await args.collection.config.hooks.beforeOperation.reduce(async (priorHook, hook) => {
        await priorHook;
        args = (await hook({
            args,
            operation: 'create',
        })) || args;
    }, Promise.resolve());
    const { collection: { Model, config: collectionConfig, }, req, req: { locale, fallbackLocale, }, disableVerificationEmail, depth, overrideAccess, showHiddenFields, } = args;
    let { data } = args;
    // /////////////////////////////////////
    // Access
    // /////////////////////////////////////
    if (!overrideAccess) {
        await executeAccess_1.default({ req }, collectionConfig.access.create);
    }
    // /////////////////////////////////////
    // beforeValidate - Fields
    // /////////////////////////////////////
    data = await this.performFieldOperations(collectionConfig, {
        data,
        req,
        hook: 'beforeValidate',
        operation: 'create',
        overrideAccess,
    });
    // /////////////////////////////////////
    // beforeValidate - Collections
    // /////////////////////////////////////
    await collectionConfig.hooks.beforeValidate.reduce(async (priorHook, hook) => {
        await priorHook;
        data = (await hook({
            data,
            req,
            operation: 'create',
        })) || data;
    }, Promise.resolve());
    // /////////////////////////////////////
    // beforeChange - Fields
    // /////////////////////////////////////
    data = await performFieldOperations(collectionConfig, {
        data,
        hook: 'beforeChange',
        operation: 'create',
        req,
        overrideAccess,
    });
    // /////////////////////////////////////
    // beforeChange - Collection
    // /////////////////////////////////////
    await collectionConfig.hooks.beforeChange.reduce(async (priorHook, hook) => {
        await priorHook;
        data = (await hook({
            data,
            req,
            operation: 'create',
        })) || data;
    }, Promise.resolve());
    // /////////////////////////////////////
    // Upload and resize potential files
    // /////////////////////////////////////
    if (collectionConfig.upload) {
        const fileData = {};
        const { staticDir, imageSizes } = collectionConfig.upload;
        const file = ((req.files && req.files.file) ? req.files.file : req.file);
        if (!file) {
            throw new errors_1.MissingFile();
        }
        let staticPath = staticDir;
        if (staticDir.indexOf('/') !== 0) {
            staticPath = path_1.default.join(config.paths.configDir, staticDir);
        }
        mkdirp_1.default.sync(staticPath);
        const fsSafeName = await getSafeFilename_1.default(staticPath, file.name);
        try {
            await file.mv(`${staticPath}/${fsSafeName}`);
            if (isImage_1.default(file.mimetype)) {
                const dimensions = await getImageSize_1.default(`${staticPath}/${fsSafeName}`);
                fileData.width = dimensions.width;
                fileData.height = dimensions.height;
                if (Array.isArray(imageSizes) && file.mimetype !== 'image/svg+xml') {
                    fileData.sizes = await imageResizer_1.default(staticPath, collectionConfig, fsSafeName, fileData.mimeType);
                }
            }
        }
        catch (err) {
            console.error(err);
            throw new errors_1.FileUploadError();
        }
        fileData.filename = fsSafeName;
        fileData.filesize = file.size;
        fileData.mimeType = file.mimetype;
        data = {
            ...data,
            ...fileData,
        };
    }
    // /////////////////////////////////////
    // Create
    // /////////////////////////////////////
    let doc = new Model();
    if (locale && doc.setLocale) {
        doc.setLocale(locale, fallbackLocale);
    }
    if (collectionConfig.auth) {
        if (data.email) {
            data.email = data.email.toLowerCase();
        }
        if (collectionConfig.auth.verify) {
            data._verified = false;
            data._verificationToken = crypto_1.default.randomBytes(20).toString('hex');
        }
    }
    Object.assign(doc, data);
    if (collectionConfig.auth) {
        doc = await Model.register(doc, data.password);
    }
    else {
        await doc.save();
    }
    let result = doc.toJSON({ virtuals: true });
    result = removeInternalFields_1.default(result);
    result = JSON.stringify(result);
    result = JSON.parse(result);
    // /////////////////////////////////////
    // afterChange - Fields
    // /////////////////////////////////////
    result = await performFieldOperations(collectionConfig, {
        data: result,
        hook: 'afterChange',
        operation: 'create',
        req,
        depth,
        overrideAccess,
        showHiddenFields,
    });
    // /////////////////////////////////////
    // afterChange - Collection
    // /////////////////////////////////////
    await collectionConfig.hooks.afterChange.reduce(async (priorHook, hook) => {
        await priorHook;
        result = await hook({
            doc: result,
            req: args.req,
            operation: 'create',
        }) || result;
    }, Promise.resolve());
    // /////////////////////////////////////
    // Send verification email if applicable
    // /////////////////////////////////////
    if (collectionConfig.auth && collectionConfig.auth.verify) {
        sendVerificationEmail_1.default({
            emailOptions,
            config: this.config,
            sendEmail: this.sendEmail,
            collection: { config: collectionConfig, Model },
            user: result,
            token: data._verificationToken,
            req,
            disableEmail: disableVerificationEmail,
        });
    }
    // /////////////////////////////////////
    // afterRead - Fields
    // /////////////////////////////////////
    result = await this.performFieldOperations(collectionConfig, {
        depth,
        req,
        data: result,
        hook: 'afterRead',
        operation: 'create',
        overrideAccess,
        reduceLocales: false,
        showHiddenFields,
    });
    // /////////////////////////////////////
    // afterRead - Collection
    // /////////////////////////////////////
    await collectionConfig.hooks.afterRead.reduce(async (priorHook, hook) => {
        await priorHook;
        result = await hook({
            req,
            doc: result,
        }) || result;
    }, Promise.resolve());
    // /////////////////////////////////////
    // Return results
    // /////////////////////////////////////
    return result;
}
exports.default = create;
