"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-use-before-define */
const graphql_1 = require("graphql");
const graphql_type_json_1 = require("graphql-type-json");
const withNullableType_1 = __importDefault(require("./withNullableType"));
const formatName_1 = __importDefault(require("../utilities/formatName"));
const combineParentName_1 = __importDefault(require("../utilities/combineParentName"));
function buildMutationInputType(name, fields, parentName, forceNullable = false) {
    const fieldToSchemaMap = {
        number: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLFloat, forceNullable) }),
        text: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        email: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        textarea: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        richText: (field) => ({ type: withNullableType_1.default(field, graphql_type_json_1.GraphQLJSON, forceNullable) }),
        code: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        date: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        upload: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        'rich-text': (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        html: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        radio: (field) => ({ type: withNullableType_1.default(field, graphql_1.GraphQLString, forceNullable) }),
        checkbox: () => ({ type: graphql_1.GraphQLBoolean }),
        select: (field) => {
            const formattedName = `${combineParentName_1.default(parentName, field.name)}_MutationInput`;
            let type = new graphql_1.GraphQLEnumType({
                name: formattedName,
                values: field.options.reduce((values, option) => {
                    if (typeof option === 'object' && option.value) {
                        return {
                            ...values,
                            [formatName_1.default(option.value)]: {
                                value: option.value,
                            },
                        };
                    }
                    if (typeof option === 'string') {
                        return {
                            ...values,
                            [option]: {
                                value: option,
                            },
                        };
                    }
                    return values;
                }, {}),
            });
            type = field.hasMany ? new graphql_1.GraphQLList(type) : type;
            type = withNullableType_1.default(field, type, forceNullable);
            return { type };
        },
        relationship: (field) => {
            const { relationTo } = field;
            let type = graphql_1.GraphQLString;
            if (Array.isArray(relationTo)) {
                const fullName = `${combineParentName_1.default(parentName, field.label)}RelationshipInput`;
                type = new graphql_1.GraphQLInputObjectType({
                    name: fullName,
                    fields: {
                        relationTo: {
                            type: new graphql_1.GraphQLEnumType({
                                name: `${fullName}RelationTo`,
                                values: relationTo.reduce((values, option) => ({
                                    ...values,
                                    [formatName_1.default(option)]: {
                                        value: option,
                                    },
                                }), {}),
                            }),
                        },
                        value: { type: graphql_1.GraphQLString },
                    },
                });
            }
            return { type: field.hasMany ? new graphql_1.GraphQLList(type) : type };
        },
        array: (field) => {
            const fullName = combineParentName_1.default(parentName, field.label);
            let type = buildMutationInputType(fullName, field.fields, fullName);
            type = new graphql_1.GraphQLList(withNullableType_1.default(field, type, forceNullable));
            return { type };
        },
        group: (field) => {
            const requiresAtLeastOneField = field.fields.some((subField) => (subField.required && !subField.localized));
            const fullName = combineParentName_1.default(parentName, field.label);
            let type = buildMutationInputType(fullName, field.fields, fullName);
            if (requiresAtLeastOneField)
                type = new graphql_1.GraphQLNonNull(type);
            return { type };
        },
        blocks: () => ({ type: graphql_type_json_1.GraphQLJSON }),
        row: (field) => field.fields.reduce((acc, rowField) => {
            const getFieldSchema = fieldToSchemaMap[rowField.type];
            if (getFieldSchema) {
                const fieldSchema = getFieldSchema(rowField);
                return [
                    ...acc,
                    fieldSchema,
                ];
            }
            return null;
        }, []),
    };
    const fieldTypes = fields.reduce((schema, field) => {
        if (!field.hidden) {
            const getFieldSchema = fieldToSchemaMap[field.type];
            if (getFieldSchema) {
                const fieldSchema = getFieldSchema(field);
                if (Array.isArray(fieldSchema)) {
                    return fieldSchema.reduce((acc, subField, i) => ({
                        ...acc,
                        [field.fields[i].name]: subField,
                    }), schema);
                }
                return {
                    ...schema,
                    [field.name]: fieldSchema,
                };
            }
        }
        return schema;
    }, {});
    const fieldName = formatName_1.default(name);
    return new graphql_1.GraphQLInputObjectType({
        name: `mutation${fieldName}Input`,
        fields: {
            ...fieldTypes,
        },
    });
}
exports.default = buildMutationInputType;
